//***********************************************************
//
//  NXV4 "Math-on-Paper" software
//
//  package : CPX class _____________________________________
//            
//            supports: move semantix
//                      operators + - * / ^   L,R-val context
//                                += -= *= /=
//                      fabs()
//                      phi ()    [ 0  ... 2.pi] o/p
//                      phi2()    [-pi ...  +pi] o/p
//                      ~  -      unary operators L,R-val
//                      cpx<arithmetic> i.e. int
//                                           float
//                                           double
//                                           long double
//                      type_casts between the types 
//                      cout << overload 
//
//                      TOTAL = 27+1205 instantiated op/f's
//                      _____________________________________
//
//
//  authors : Mihai-Tiberiu Dima (1)
//            Maria Dima         (2)
//
//             Meshcheryakov Laboratory of Information Techn.
//             Joint Institute for Nuclear Research
//             Dubna-Russia
//
//            on leave from:
//
//             Hyperion Univ. Bucharest-Romania
//
//
//  date    : CPX4/v1.0.alfa   / Mon Dec 27 17:13:57 CET 2021
//                 v2.1.beta   / Mon Mar 07 12:45:54 CET 2022
//
//***********************************************************
#include <cmath>
#include <iostream>

#include "cpx.hh"

using namespace std                                         ;
using namespace cpx4                                        ;


int main()
{

// this is a function foliage example
//
//   sqrt(1+0.001i) = approx. +1
//   sqrt(1-0.001i) = approx. -1
//
// to use this example do the following:
//   rm test.cc
//   ln -s e1.cc test.cc
//   make test
//   make run

  using dblx = double                                       ;
  using real = long double                                  ;

  double e   = exp(1)                                       ;
  double pi  = 3.14159265853                                ;

  cpx<int >  i__int(0,1)                                    ;
  cpx<dblx>  i_dblx(0,1)                                    ;
  cpx<real>  i_real(0,1)                                    ;

  int        u__int = 1                                     ;
  dblx       u_dblx = 1                                     ;
  int        u_real = 1                                     ;



  cout                                             << endl  ;
  cout << boolalpha                                << endl  ;



  auto       z = u_dblx + i__int                            ;
  cout                                             << endl  ;
  cout << " _____________________________________" << endl  ;
  cout                                             << endl  ;
  cout << "   " << "sqrt(1+0.001i) = "             <<
                    sqrt(cpx<double>(1, 0.001) )   << endl  ;
  cout                                             << endl  ;
  cout << "   " << "sqrt(1-0.001i) = "             <<
                    sqrt(cpx<double>(1,-0.001) )   << endl  ;
  cout << " _____________________________________" << endl  ;
  cout                                             << endl  ;
  z = exp(cpx<double>(1, 0.001))                            ;
  cout << "   " << "log(exp(1+0.001i)) = "         <<
                    log(z)                         << endl  ;
  cout                                             << endl  ;
  z = exp(cpx<double>(1,-0.001))                            ;
  cout << "   " << "log(exp(1-0.001i)) = "         <<
                    log(z)                         << endl  ;
  cout                                             << endl  ;
  cout << "   Note: that imag. part = 2*pi, as it" << endl  ;
  cout << "         should be (correct foliage)"   << endl  ;
  cout << " _____________________________________" << endl  ;
  cout                                             << endl  ;
  cout                                             << endl  ;
  cout                                             << endl  ;

}
