//***********************************************************
//
//  NXV4 "Math-on-Paper" software
//
//  package : SU2 class _____________________________________
//            
//            supports: move semantix
//                      operators + - * / ^   L,R-val context
//                                += -= *= /=
//                      fabs()
//                      ~  -      unary operators L,R-val
//                      su2<arithmetic> i.e.     int
//                                               float 
//                                               double
//                                               real
//                                           cpx<int> 
//                                           cpx<float> 
//                                           cpx<double> 
//                                           cpx<real> 
//                      type_casts between the types 
//                      cout << overload ____________________
//
//
//                      Note: zz = z - tr(z)/2
//                            sg = zz/fabs(z)
//
//                      Equivalent results:
//
//                        sqrt -> +/- sqrt * sg
//                             -> +/- sqrt
//
//                        log  -> log  + 2*n*pi*i       +
//                                       2*m*pi*i* sg   +
//                                         k*pi*i*(sg+1)
//
//
//                      TOTAL = 5668 instantiated op/f's
//                      _____________________________________
//
//
//  authors : Mihai-Tiberiu Dima 
//            Maria Dima 
//
//  scientif.
//  coord   : Mihai Dima
//
//            Meshcheryakov Laboratory of Information Techn.
//            Joint Institute for Nuclear Research
//            Dubna-Russia
//
//
//  date    : SU24/v1.0.alfa   / Mon Mar 28 15:17:02 CET 2022
//
//***********************************************************
#include <cmath>
#include <iostream>

#include "cpx.hh"
#include "vu2.hh"
#include "su2.hh"

using namespace std                                         ;
using namespace cpx4                                        ;
using namespace vu24                                        ;
using namespace su24                                        ;


int main()
{

// this is the diadic product of two
// wave-vectors
//
//   |vu2><vu2> = operator
//     ^   ^ 
//     |   |_cpx<int>
//     |
//     |_dblx
//
// to use this example do the following:
//   rm test.cc
//   ln -s e1.cc test.cc
//   make test
//   make run

  using fltx = float                                        ;
  using dblx = double                                       ;
  using real = long double                                  ;

  double e   = exp(1)                                       ;
  double pi  = 3.14159265853                                ;

  cpx<int >  i(0,1)                                         ;

  su2<dblx> sx(0,1,
               1,0 )                                        ;

  vu2<cpx<int>> phi(0,i)                                    ;
  vu2<double  > psi(1,1)                                    ;


  cout                                             << endl  ;
  cout << noboolalpha                              << endl  ;

  cout                                             << endl  ;
  cout << " _____________________________________" << endl  ;
  cout                                             << endl  ;
  cout << "   " << noboolalpha
                << "( " << phi << " ) ( "
                << (~psi)  << " ) = " << (phi^psi) << endl  ;
  cout                                             << endl  ;
  cout << " _____________________________________" << endl  ;
  cout                                             << endl  ;
  cout                                             << endl  ;
  cout                                             << endl  ;

}
