//***********************************************************
//
//  NXV4 "Math-on-Paper" software
//
//  package : SU2 class _____________________________________
//            
//            supports: move semantix
//                      operators + - * / ^   L,R-val context
//                                += -= *= /=
//                      fabs()
//                      ~  -      unary operators L,R-val
//                      su2<arithmetic> i.e.     int
//                                               float 
//                                               double
//                                               real
//                                           cpx<int> 
//                                           cpx<float> 
//                                           cpx<double> 
//                                           cpx<real> 
//                      type_casts between the types 
//                      cout << overload ____________________
//
//
//                      Note: zz = z - tr(z)/2
//                            sg = zz/fabs(z)
//
//                      Equivalent results:
//
//                        sqrt -> +/- sqrt * sg
//                             -> +/- sqrt
//
//                        log  -> log  + 2*n*pi*i       +
//                                       2*m*pi*i* sg   +
//                                         k*pi*i*(sg+1)
//
//
//                      TOTAL = 5668 instantiated op/f's
//                      _____________________________________
//
//
//  authors : Mihai-Tiberiu Dima 
//            Maria Dima 
//
//  scientif.
//  coord   : Mihai Dima
//
//            Meshcheryakov Laboratory of Information Techn.
//            Joint Institute for Nuclear Research
//            Dubna-Russia
//
//
//  date    : SU24/v1.0.alfa   / Mon Mar 28 15:17:02 CET 2022
//
//***********************************************************
#pragma once
#pragma once


#include<iostream>
#include<cmath>
#include<type_traits>
#include<utility>

#include "cpx.hh"
#include "vu2.hh"

using namespace cpx4                                        ;
using namespace vu24                                        ;

using std::cout                                             ;
using std::endl                                             ;
using std::decay                                            ;
using std::declval                                          ;
using std::enable_if                                        ;
using std::is_same                                          ;
using std::forward                                          ;
using std::ostream                                          ;
using std::integral_constant                                ;
using std::remove_cv                                        ;



namespace su24
{

 bool  utf8x = false                                        ; 

 using fltx =      float                                    ;
 using dblx =      double                                   ;
 using real = long double                                   ;

template<typename T>
struct is_cscalar: integral_constant<bool, 
 is_same<    int         , typename remove_cv<T>::type>::value ||
 is_same<    fltx        , typename remove_cv<T>::type>::value ||
 is_same<    dblx        , typename remove_cv<T>::type>::value ||
 is_same<    real        , typename remove_cv<T>::type>::value ||
 is_same<cpx<int >       , typename remove_cv<T>::type>::value ||
 is_same<cpx<fltx>       , typename remove_cv<T>::type>::value ||
 is_same<cpx<dblx>       , typename remove_cv<T>::type>::value ||
 is_same<cpx<real>       , typename remove_cv<T>::type>::value  >
{                                                           ;};


 template<typename scalar=dblx>
 class su2
   {public: typename decay<scalar>::type x11 = 0            ;
            typename decay<scalar>::type x12 = 0            ;
            typename decay<scalar>::type x21 = 0            ;
            typename decay<scalar>::type x22 = 0            ;
      // 
      su2()                         // plain     
       {                                                    ;} 
      //
      su2(     scalar a )           // manual     
       {x11 = a                                             ;
        x12 = 0                                             ;
        x21 = 0                                             ;
        x22 = a                                             ;} 
      //
      su2(scalar a11, scalar a12,
          scalar a21, scalar a22 )  // manual     
       {x11 = a11                                           ;
        x12 = a12                                           ;
        x21 = a21                                           ;
        x22 = a22                                           ;}
      //
      su2(const su2& A)             // copy       
       {x11 = A.x11                                         ;
        x12 = A.x12                                         ;
        x21 = A.x21                                         ;
        x22 = A.x22                                         ;}

      template<typename U>
      su2(U const& a11, U const& a12,
          U const& a21, U const& a22 )  // manual     
       {x11 = scalar(const_cast<U&>(a11))                   ;
        x12 = scalar(const_cast<U&>(a12))                   ;
        x21 = scalar(const_cast<U&>(a21))                   ;
        x22 = scalar(const_cast<U&>(a22))                   ;
       }

      template<typename U>
      su2(su2<U> const& A)
       {x11 = scalar(const_cast<U&>(A.x11))                 ;
        x12 = scalar(const_cast<U&>(A.x12))                 ;
        x21 = scalar(const_cast<U&>(A.x21))                 ;
        x22 = scalar(const_cast<U&>(A.x22))                 ;
       }

      // CAST  _____________________________________________

      explicit
      operator     int  ()         
      {return      int  (int((x11+x22)/2))                  ;}

      explicit
      operator     fltx ()         
      {return      fltx (fltx((x11+x22)/2))                 ;}

      explicit
      operator     dblx ()         
      {return      dblx (dblx((x11+x22)/2))                 ;}

      explicit
      operator     real ()         
      {return      real (real((x11+x22)/2))                 ;}

      explicit
      operator cpx<int >()         
      {return  cpx<int >(int((x11+x22)/2))                  ;}

      explicit
      operator cpx<fltx>()         
      {return  cpx<fltx>(fltx((x11+x22)/2))                 ;}

      explicit
      operator cpx<dblx>()         
      {return  cpx<dblx>(dblx((x11+x22)/2))                 ;}

      explicit
      operator cpx<real>()         
      {return  cpx<real>(real((x11+x22)/2))                 ;}

      explicit
      operator su2<int >()
      {return  su2<int >(int (x11), int (x12),
                         int (x21), int (x22) )             ;}
      explicit
      operator su2<fltx>()
      {return  su2<fltx>(fltx(x11), fltx(x12),
                         fltx(x21), fltx(x22) )             ;}
      explicit
      operator su2<dblx>()
      {return  su2<dblx>(dblx(x11), dblx(x12),
                         dblx(x21), dblx(x22) )             ;}
      explicit
      operator su2<real>()
      {return  su2<real>(real(x11), real(x12),
                         real(x21), real(x22) )             ;}

      explicit
      operator su2<cpx<int >>()
      {return  su2<cpx<int >>(cpx<int >(x11), cpx<int >(x12),
                              cpx<int >(x21), cpx<int >(x22) );}
      explicit
      operator su2<cpx<fltx>>()
      {return  su2<cpx<fltx>>(cpx<fltx>(x11), cpx<fltx>(x12),
                              cpx<fltx>(x21), cpx<fltx>(x22) );}
      explicit
      operator su2<cpx<dblx>>()
      {return  su2<cpx<dblx>>(cpx<dblx>(x11), cpx<dblx>(x12),
                              cpx<dblx>(x21), cpx<dblx>(x22) );}
      explicit
      operator su2<cpx<real>>()
      {return  su2<cpx<real>>(cpx<real>(x11), cpx<real>(x12),
                              cpx<real>(x21), cpx<real>(x22) );}

      // ASSGN _____________________________________________

      scalar& operator= (const scalar& A)
       {x11 = A                                             ;
        x12 = 0                                             ;
        x21 = 0                                             ;
        x22 = A                                             ;
        return const_cast<scalar&>(A)                       ;}

         su2& operator= (const    su2& A)
       {x11 = A.x11                                         ;
        x12 = A.x12                                         ;
        x21 = A.x21                                         ;
        x22 = A.x22                                         ;
        return const_cast<su2&>(A)                          ;}

      template<typename U> 
      su2<U>& operator= (su2<U> const& A)
       {x11 = scalar(const_cast<U&>(A.x11))                 ;
        x12 = scalar(const_cast<U&>(A.x12))                 ;
        x21 = scalar(const_cast<U&>(A.x21))                 ;
        x22 = scalar(const_cast<U&>(A.x22))                 ;
        return const_cast<su2<U>&>(A)                       ;}

      // NGTV ______________________________________________

      template<typename T>
      friend
      su2<T>
      operator- (su2<T> const& A)                           ;

      template<typename T>
      friend
      su2<T>&&
      operator- (su2<T> && A)                               ;

      // TILDE (su2.conj.) _________________________________

      template<typename T>
      friend
      su2<T>
      operator~ (su2<T> const& A)                           ;

      template<typename T>
      friend
      su2<T>&&
      operator~ (su2<T> && A)                               ;

      // FABS ______________________________________________

      template<typename T>
      friend
      long double fabs(su2<T> const& A)                     ;

      template<typename T>
      friend
      long double n2(su2<T> const& A)                       ;

      // PROD ______________________________________________

      template<typename L, typename R>
      friend
      typename decay<decltype(declval<L>()*
                              declval<R>())>::type
      operator|(su2<L> const& A, su2<R> const& B)           ;

      template<typename ALX, typename BLX>
      friend
      vu2<typename decay<decltype(declval<ALX>()+
                                  declval<BLX>() )
                         >::type
         >
      operator|(vu2<ALX> const& a, su2<BLX> const& b)       ;

      template<typename ALX, typename BLX>
      friend
      vu2<typename decay<decltype(declval<ALX>()+
                                  declval<BLX>() )
                         >::type
         >
      operator|(su2<ALX> const& a, vu2<BLX> const& b)       ;

      // COEF's _____________________________________________

      template<typename T>
      friend
      cpx<real> sigma1(su2<T> const& A)                     ;

      template<typename T>
      friend
      cpx<real> sigmax(su2<T> const& A)                     ;

      template<typename T>
      friend
      cpx<real> sigmay(su2<T> const& A)                     ;

      template<typename T>
      friend
      cpx<real> sigmaz(su2<T> const& A)                     ;

      template<typename T>
      friend
      cpx<real> tr(su2<T> const& A)                         ;

      template<typename T>
      friend
      cpx<real> det(su2<T> const& A)                        ;

      // OPERATORS __________________________________________

         #include "eqal.cxx"
         #include "comp.cxx"

         #include "plus.cxx"
         #include "mnus.cxx"
         #include "mult.cxx"
         #include "divd.cxx"

         #include "alog.cxx"
         #include "expo.cxx"

      // PRINT _____________________________________________

      friend std::ostream& operator<<(std::ostream&   , 
                                      su2<auto> const& )    ;

      // ___________________________________________________

     ~su2()
       {                                                    ;}
 };

//===========================================================
};
