//***********************************************************
//
//  NXV4 "Math-on-Paper" software
//
//  package : SU2 class _____________________________________
//            
//            supports: move semantix
//                      operators + - * / ^   L,R-val context
//                                += -= *= /=
//                      fabs()
//                      ~  -      unary operators L,R-val
//                      su2<arithmetic> i.e.     int
//                                               float 
//                                               double
//                                               real
//                                           cpx<int> 
//                                           cpx<float> 
//                                           cpx<double> 
//                                           cpx<real> 
//                      type_casts between the types 
//                      cout << overload ____________________
//
//
//                      Note: zz = z - tr(z)/2
//                            sg = zz/fabs(z)
//
//                      Equivalent results:
//
//                        sqrt -> +/- sqrt * sg
//                             -> +/- sqrt
//
//                        log  -> log  + 2*n*pi*i       +
//                                       2*m*pi*i* sg   +
//                                         k*pi*i*(sg+1)
//
//
//                      TOTAL = 5668 instantiated op/f's
//                      _____________________________________
//
//
//  authors : Mihai-Tiberiu Dima 
//            Maria Dima 
//
//  scientif.
//  coord   : Mihai Dima
//
//            Meshcheryakov Laboratory of Information Techn.
//            Joint Institute for Nuclear Research
//            Dubna-Russia
//
//
//  date    : SU24/v1.0.alfa   / Mon Mar 28 15:17:02 CET 2022
//
//***********************************************************
#include <cmath>
#include <iostream>

#include "cpx.hh"
#include "vu2.hh"
#include "su2.hh"

using namespace std                                         ;
using namespace cpx4                                        ;
using namespace vu24                                        ;
using namespace su24                                        ;


// application for solution to telegrapher's
// equation
//
// d/dx + (sx d/d(ct) + 1/La) + jsy/Ld = 0| vu2
//
// where:
//  - sx = Pauli mtx sigma_x
//  - sy = Pauli mtx sigma_y
//  -  c = speed of transmission
//  - La = attenuation length
//  - Ld = dispersion  length
//  -  j = cpx<double>(0,1)
//  - vu2= solution
//
// to use this example do the following:
//   rm test.cc
//   ln -s e1.cc test.cc
//   make test
//   make run

  using fltx = float                                        ;
  using dblx = double                                       ;
  using real = long double                                  ;

 real  Ld   = 200.0                                         ;
 real  c    =   3.0e8                                       ;
 real    pi = 3.14159265358979323846264L                    ;
 real    e  = 2.71828182845904523536028L                    ;

 auto sx = su2<real>     (0,  1,
                          1,  0)                            ;

 auto sy = su2<real>     (0,  1,
                         -1,  0)                            ;

 auto sz = su2<real>     (1,  0,
                          0, -1)                            ;

auto propagator(real x, 
                real t,
                real f )
{
 real gamma = sqrt(1+f*f*Ld*Ld/c/c)                         ;
 real beta  = sqrt(gamma*gamma-1) / gamma                   ;
 return e^(-(1+sx*beta)*sy*(x-beta*c*t)*gamma*gamma/Ld)     ;}


int main()
{

 real f0 =   0.004                                          ;
 real tt = 100.000                                          ;
 real tr =  10.000                                          ;

 int  N0 =  -10                                             ;
 int  N  = 9000 + N0                                        ;

 for (int i=0; i<N; i++) {
  cout << i << " "
            <<((propagator(N0,i, 1*f0)/ 1*sin( 1*pi*f0*370)+
                propagator(N0,i, 3*f0)/ 3*sin( 3*pi*f0*370)+
                propagator(N0,i, 5*f0)/ 5*sin( 5*pi*f0*370)+
                propagator(N0,i, 7*f0)/ 7*sin( 7*pi*f0*370)+
                propagator(N0,i, 9*f0)/ 9*sin( 9*pi*f0*370)+
                propagator(N0,i,11*f0)/11*sin(11*pi*f0*370)+
                propagator(N0,i,13*f0)/13*sin(13*pi*f0*370)+
                propagator(N0,i,15*f0)/15*sin(15*pi*f0*370))|
                vu2<dblx>(1,0)).y
                << endl                                     ;}
}
