//***********************************************************
//
//  NXV4 "Math-on-Paper" software
//
//  package : PHI class _____________________________________
//            
//            supports: move semantix
//                      operators + - * / ^   L,R-val context
//                                += -= *= /=
//                      fabs()
//                      ~  -      unary operators L,R-val
//                      vu2<arithmetic> i.e.     int
//                                               float 
//                                               double
//                                               real
//                                           cpx<int> 
//                                           cpx<float> 
//                                           cpx<double> 
//                                           cpx<real> 
//                      type_casts between the types 
//                      cout << overload 
//
//                      TOTAL = 2452 instantiated op/f's
//                      _____________________________________
//
//
//  authors : Mihai-Tiberiu Dima 
//            Maria Dima 
//
//  scientif.
//  coord's : Mihai Dima
//            Gheorghe Adam
//
//            Meshcheryakov Laboratory of Information Techn.
//            Joint Institute for Nuclear Research
//            Dubna-Russia
//
//
//  date    : PHI4/v1.0.alfa   / Mon Mar 28 15:17:02 CET 2022
//
//***********************************************************
#include <cmath>
#include <iostream>

#include "cpx.hh"
#include "vu2.hh"

using namespace std                                         ;
using namespace cpx4                                        ;
using namespace vu24                                        ;


int main()
{

 vu24::utf8x = true                                         ;

// this is an algebra example
//
//   |vu2> =  |up> + 2*i*|down>
//   |vu2> = -|up> + 3*i*|down>
//
//   <vu2|vu2> = -1 + (-2i)*(3i) = 5
//
// to use this example do the following:
//   rm test.cc
//   ln -s e1.cc test.cc
//   make test
//   make run

  using fltx = float                                        ;
  using dblx = double                                       ;
  using real = long double                                  ;

  double e   = exp(1)                                       ;
  double pi  = 3.14159265853                                ;

  cpx<dblx>  i(0,1)                                         ;

  auto phi = vu2<int>( 1,0) + 2*i*vu2<dblx>(0,1)            ; 
  auto psi = vu2<int>(-1,0) + vu2<cpx<dblx>>(0,3*i)         ; 

  cout                                             << endl  ;
  cout << noboolalpha                              << endl  ;

  cout                                             << endl  ;
  cout << " _____________________________________" << endl  ;
  cout                                             << endl  ;
  cout << "   " << "( "    << phi 
                << " | "   << psi
                << " ) = " << boolalpha
                <<            (phi|psi)            << endl  ;
  cout                                             << endl  ;
  cout << " _____________________________________" << endl  ;
  cout                                             << endl  ;
  cout                                             << endl  ;
  cout                                             << endl  ;

}
