//***********************************************************
//
//  NXV4 "Math-on-Paper" software
//
//  package : VU2 class _____________________________________
//            
//            supports: move semantix
//                      operators + - * / ^   L,R-val context
//                                += -= *= /=
//                      fabs()
//                      ~  -      unary operators L,R-val
//                      vu2<arithmetic> i.e.     int
//                                               float 
//                                               double
//                                               real
//                                           cpx<int> 
//                                           cpx<float> 
//                                           cpx<double> 
//                                           cpx<real> 
//                      type_casts between the types 
//                      cout << overload 
//
//                      TOTAL = 2452 instantiated op/f's
//                      _____________________________________
//
//
//  authors : Mihai-Tiberiu Dima 
//            Maria Dima 
//
//  scientif.
//  coord's : Mihai Dima
//            Gheorghe Adam
//
//            Meshcheryakov Laboratory of Information Techn.
//            Joint Institute for Nuclear Research
//            Dubna-Russia
//
//
//  date    : VU24/v1.0.alfa   / Mon Mar 28 15:17:02 CET 2022
//
//***********************************************************
#pragma once
#pragma once


#include<iostream>
#include<cmath>
#include<type_traits>
#include<utility>

#include "cpx.hh"

using namespace cpx4                                        ;

using std::cout                                             ;
using std::endl                                             ;
using std::decay                                            ;
using std::declval                                          ;
using std::enable_if                                        ;
using std::is_same                                          ;
using std::forward                                          ;
using std::ostream                                          ;
using std::integral_constant                                ;
using std::remove_cv                                        ;

namespace su24
{
 template<typename T>
 class su2                                                  ;
}
using namespace su24                                        ;

namespace vu24
{

 bool  utf8x = false                                        ; 

 using fltx  =      float                                   ;
 using dblx  =      double                                  ;
 using real  = long double                                  ;

template< class T >
struct is_cscalar: integral_constant<bool, 
 is_same<    int         , typename remove_cv<T>::type>::value ||
 is_same<    fltx        , typename remove_cv<T>::type>::value ||
 is_same<    dblx        , typename remove_cv<T>::type>::value ||
 is_same<    real        , typename remove_cv<T>::type>::value ||
 is_same<cpx<int >       , typename remove_cv<T>::type>::value ||
 is_same<cpx<fltx>       , typename remove_cv<T>::type>::value ||
 is_same<cpx<dblx>       , typename remove_cv<T>::type>::value ||
 is_same<cpx<real>       , typename remove_cv<T>::type>::value  >
{                                                           ;};


 template<typename scalar=dblx>
 class vu2
   {public: typename decay<scalar>::type x = 0              ;
            typename decay<scalar>::type y = 0              ;
                               bool rangle = true           ;
      // 
      vu2()                         // plain     
       {                                                    ;} 
      //
      vu2(     scalar a )           // manual     
       {x = a                                               ;
        y = a                                               ;} 
      //
      vu2(scalar a, scalar aa)      // manual     
       {x = aa                                              ;
        y =  a                                              ;}
      //
      vu2(const vu2& A)             // copy       
       {x = A.x                                             ;
        y = A.y                                             ;
        rangle = A.rangle                                   ;}

      template<typename U>
      vu2(U const& a, U const& aa)
       {x = scalar(const_cast<U&>(aa))                      ;
        y = scalar(const_cast<U&>(a ))                      ;
       }

      template<typename U>
      vu2(vu2<U> const& A)
       {x = scalar(const_cast<U&>(A.x))                     ;
        y = scalar(const_cast<U&>(A.y))                     ;
        rangle = A.rangle                                   ;}

      // CAST  _____________________________________________

      explicit
      operator vu2<int >()
      {return  vu2<int >(int (x), int (y))                  ;}
      explicit
      operator vu2<fltx>()
      {return  vu2<fltx>(fltx(x), fltx(y))                  ;}
      explicit
      operator vu2<dblx>()
      {return  vu2<dblx>(dblx(x), dblx(y))                  ;}
      explicit
      operator vu2<real>()
      {return  vu2<real>(real(x), real(y))                  ;}

      explicit
      operator vu2<cpx<int >>()
      {return  vu2<cpx<int >>(cpx<int >(x), cpx<int >(y))   ;}
      explicit
      operator vu2<cpx<fltx>>()
      {return  vu2<cpx<fltx>>(cpx<fltx>(x), cpx<fltx>(y))   ;}
      explicit
      operator vu2<cpx<dblx>>()
      {return  vu2<cpx<dblx>>(cpx<dblx>(x), cpx<dblx>(y))   ;}
      explicit
      operator vu2<cpx<real>>()
      {return  vu2<cpx<real>>(cpx<real>(x), cpx<real>(y))   ;}

      // ASSGN _____________________________________________

      scalar& operator= (const scalar& A)
       {x = A                                               ;
        y = A                                               ;
        return const_cast<scalar&>(A)                       ;}

         vu2& operator= (const    vu2& A)
       {x = A.x                                             ;
        y = A.y                                             ;
        rangle = A.rangle                                   ;
        return const_cast<vu2&>(A)                          ;}

      template<typename U> 
      vu2<U>& operator= (vu2<U> const& A)
       {x = scalar(const_cast<U&>(A.x))                     ;
        y = scalar(const_cast<U&>(A.y))                     ;
        rangle = A.rangle                                   ;
        return const_cast<vu2<U>&>(A)                       ;}

      // NGTV ______________________________________________

      template<typename T>
      friend
      vu2<T>
      operator- (vu2<T> const& A)                           ;

      template<typename T>
      friend
      vu2<T>&&
      operator- (vu2<T> && A)                               ;

      // TILDE (vu2.conj.) _________________________________

      template<typename T>
      friend
      vu2<T>
      operator~ (vu2<T> const& A)                           ;

      template<typename T>
      friend
      vu2<T>&&
      operator~ (vu2<T> && A)                               ;

      // FABS ______________________________________________

      template<typename T>
      friend
      long double fabs(vu2<T> const& A)                     ;

      template<typename T>
      friend
      long double n2(vu2<T> const& A)                       ;

      // PROD ______________________________________________

      template<typename L, typename R>
      friend
      typename decay<decltype(declval<L>()*
                              declval<R>())>::type
      operator|(vu2<L> const& A, vu2<R> const& B)           ;

      // OPERATORS __________________________________________

         #include "eqal.cxx"
         #include "comp.cxx"

         #include "plus.cxx"
         #include "mnus.cxx"
         #include "mult.cxx"
         #include "divd.cxx"

      // ____________________________________________________

      friend std::ostream& operator<<(std::ostream&   , 
                                      vu2<auto> const& )    ;
      // ____________________________________________________

      template<typename ALX, typename BLX>
      friend
      su2<typename decay<decltype(declval<ALX>()*
                                  declval<BLX>() )
                        >::type
         >
      operator^(vu2<ALX> const& a, vu2<BLX> const& b)       ;

      // ____________________________________________________

     ~vu2()
       {                                                    ;}
 };

//===========================================================
};
